% Generation of mechanical spring model for FINELG treatment, including
% group effects and components ductility

% Conducted as part of master thesis relating to the development of an
% innovative model for the precise characterization of steel and composite
% joints

% MATHIEU Julien

close all
clear
clc

%% General data
% Lines and columns of data_all must be verified if excel file is changed

% Name of FINELG input file
Name = 'Input_Finelg';
Ext = '.dat';

% Extraction of data from excel file
data_all = xlsread('Data.xlsx');

% Name of .txt file usefull for results processing
Name_txt = 'Loads_Dep';
Ext_txt = '.txt';

% Number of rows in tension
n_rows_t = data_all(10,1);
% Number of rows in compression
n_rows_c = data_all(11,1);
% Total number of rows
n_rows_tot = n_rows_t + n_rows_c;

% Total number of groups
n_gr_tot = n_rows_t*(n_rows_t-1)/2;
% Total number of demultiplication of the rows
n_demult = n_gr_tot-(n_rows_t-1)-(n_rows_t-2)+1;

% Location of the point of application of the axial load
CG_beam = data_all(12,1);

% Bending moment
M = data_all(1,1)*10^6;
% Axial force
N = data_all(2,1)*10^3;
% Transversal load
V = data_all(3,1)*10^3;

% Initial increment
incr_ini = data_all(4,1);
% Initial small axial force (sequences 1 and 3)
N1 = data_all(5,1)*10^3;
% Initial small bending moment (sequence 2)
M1 = data_all(6,1)*10^6;

% First line of table 'Components data' in 'data_all'
first_line_comp = 2;
% First column of table 'Components data' in 'data_all'
first_column_comp = 3;
% Last line of table 'Components data' in 'data_all'
last_line_comp = size(data_all,1);

% First line of table 'Groups data' in 'data_all'
first_line_gr = 2;
% First column of table 'Groups data' in 'data_all'
first_column_gr = 11;
% Last line of table 'Groups data'
last_line_gr = first_line_gr+n_gr_tot-1;

%% Rows general data

% Column 1 : Row number
% Column 2 : Distance between the bottom row and row i
% Column 3 : Row in tension(1)/compression(0)
% Column 4 : Number of groups in row i
% Column 5 : Number of groups seperating the row i (number of "cuts")
% Column 6 : Total number of elements in row i
% Column 7 : First element in row i
% Column 8 : Last element in row i
% Column 9 : Total number of nodes in row i
% Column 10 : First node in row i
% Column 11 : Last node in row i

data_rows = zeros(n_rows_tot,4);
nt = 0;
for i = 1:n_rows_tot
    
    data_rows(i,1) = i;
    
    for j = first_line_comp:last_line_comp
        if data_all(j,first_column_comp) == i
            
            data_rows(i,2) = data_all(j,first_column_comp+1);
            data_rows(i,3) = data_all(j,first_column_comp+2);
            
        end
    end
    
    if data_rows(i,3) == 1
        nt = nt+1;
        if nt == 1
            
            data_rows(i,4) = n_rows_t-1;
            
        elseif nt <= n_demult+1
            
            n = 1;
            while data_rows(i-n,3) == 0
                n = n+1;
            end
            data_rows(i,4) = data_rows(i-n,4)-1;
            
        else
            
            n = 1;
            while data_rows(i-n,3) == 0
                n = n+1;
            end
            data_rows(i,4) = data_rows(i-n,4)-1;
            
        end
    end
end

% Number of groups considering row i in each zone
% Each zone j is composed of all equivalent springs next to groups j to i 
% Number of zones = total number of rows in tension - 3 (only one
% equivalent spring in the two first rows and no group in last row)
for i = 1:n_rows_t
    for j = 1:n_rows_t-2-1
        
        if i <= 2+(j-1)
            
            nb_groups_zone(i,j) = 0;
            
        elseif i == 2+(j-1)+1
            
            nb_groups_zone (i,j) = n_rows_t-i;
            
        elseif i == 2+(j-1)+2
            
            nb_groups_zone (i,j) = nb_groups_zone(i-1,j);
            
        else
            
            nb_groups_zone (i,j) = nb_groups_zone(i-1,j)-1;
            
        end
    end
end

% Total number of groups considering row i
n = 0;
for i = 1:n_rows_tot
    if data_rows(i,3) == 1
        n = n+1;
        data_rows(i,5) = sum(nb_groups_zone(n,:));
    end
end

n = 0;
for i = 1:n_rows_tot
    if data_rows(i,3) == 1
        n = n+1;
        
        if n <= 2
            data_rows(i,6) = data_rows(i,4)+1;
            data_rows(i,9) = data_rows(i,6)+1;
        else
            data_rows(i,6) = data_rows(i,4)+n_demult;
            data_rows(i,9) = data_rows(i,5)+n_demult+1;
        end
        
    else
        data_rows(i,6) = 1;
        data_rows(i,9) = 2;
    end
    
    data_rows(i,7) = sum(data_rows(1:i,6))-data_rows(i,6)+1;
    data_rows(i,8) = sum(data_rows(1:i,6));
    data_rows(i,10) = sum(data_rows(1:i,9))-data_rows(i,9)+1;
    data_rows(i,11) = sum(data_rows(1:i,9));
    
end

% Additional node if the last row is in tension
if data_rows(end,3) ~= 0
    data_rows(end,9) = data_rows(end,9)+1;
    data_rows(i,11) = data_rows(i,11)+1;
end

% Separation between rows in tension and rows in compression
nt = 0;
nc = 0;
for i = 1:n_rows_tot
    if data_rows(i,3) == 1
        
        nt = nt+1;
        data_rows_t(nt,:) = data_rows(i,:);
        
    else
        
        nc = nc+1;
        data_rows_c(nc,:) = data_rows(i,:);
        
    end
end

% Number of nodes
n_nodes = data_rows(end,11)+1;

% Number of linear constrains in rows in tension 3 to n_rows_t
n = 0;
for i = 3:n_rows_t
    
    n = n+1;
    lin_const_int(n,1) = sum(1:1:data_rows_t(i,4));
    
end

% Total number of linear constrains
if data_rows(end,3) == 0
    n_lin_const = sum(lin_const_int)+sum(data_rows_t(1:2,4))+(n_demult-1)*(n_rows_t-2)+n_rows_c+1;
else
    n_lin_const = sum(lin_const_int)+sum(data_rows_t(1:2,4))+(n_demult-1)*(n_rows_t-2)+n_rows_c+2;
end

% Number of fictional nodes
n_fict_nodes = ceil(n_lin_const/3);

% Total number of nodes
n_nodes_tot = n_nodes + n_fict_nodes;


%% Nodes coordinates

coord = zeros(n_nodes_tot,5);

% FINELG input
% Nodes numbering
for i = 1:n_nodes_tot
    
    coord(i,1) = i;
    
end

for i = 1:n_nodes
    
    coord(i,2) = 0;
    coord(i,3) = 0;
    coord(i,5) = 0;
    
end

% Distance from the bottom row to the node
for i = 1:n_rows_tot
    for j = data_rows(i,10):data_rows(i,11)
        
        coord(j,4) = data_rows(i,2);
        
    end
end

% Coordinates of the point of application of the axial load
coord(end-n_fict_nodes,4) = CG_beam;


%% Constitutive laws

% Constitutive laws of the components
for i = 1:n_rows_tot
    n = 0;
    for j = 1:last_line_comp-first_line_comp+1
        if data_all(first_line_comp-1+j,first_column_comp) == i
            n = n+1;
            
            % Bi-linear law
            if data_all(first_line_comp-1+j,first_column_comp+3) == 1
                
                Resistance(n,1,i) = data_all(first_line_comp-1+j,first_column_comp+5);
                Rigidity(n,1,i) = min(data_all(first_line_comp-1+j,first_column_comp+4),500000000);
                Delta(n,i) = data_all(first_line_comp-1+j,first_column_comp+6);
                
                % Tri-linear law
            elseif data_all(first_line_comp-1+j,first_column_comp+3) == 2
                
                Resistance(n,2,i) = data_all(first_line_comp-1+j,first_column_comp+5);
                Resistance(n,1,i) = Resistance(n,2,i)*2/3;
                Rigidity(n,1,i) = min(data_all(first_line_comp-1+j,first_column_comp+4),500000000);
                Rigidity(n,2,i) = (Resistance(n,2,i)-Resistance(n,1,i))/(Resistance(n,2,i)/(Rigidity(n,1,i)/3)-Resistance(n,1,i)/Rigidity(n,1,i));
                Delta(n,i) = data_all(first_line_comp-1+j,first_column_comp+6);
                
                % Multi-linear law
            elseif data_all(first_line_comp-1+j,first_column_comp+3) == 3
                
                Resistance(n,4,i) = data_all(first_line_comp-1+j,first_column_comp+5);
                Resistance(n,1,i) = Resistance(n,4,i)*6/9;
                Resistance(n,2,i) = Resistance(n,4,i)*7/9;
                Resistance(n,3,i) = Resistance(n,4,i)*8/9;
                Rigidity(n,1,i) = min(data_all(first_line_comp-1+j,first_column_comp+4),500000000);
                Rigidity(n,2,i) = (Resistance(n,2,i)-Resistance(n,1,i))/(Resistance(n,2,i)/(Rigidity(n,1,i)/1.5)-Resistance(n,1,i)/Rigidity(n,1,i));
                Rigidity(n,3,i) = (Resistance(n,3,i)-Resistance(n,2,i))/(Resistance(n,3,i)/(Rigidity(n,1,i)/2.2)-Resistance(n,1,i)/(Rigidity(n,1,i))/1.5);
                Rigidity(n,4,i) = (Resistance(n,4,i)-Resistance(n,3,i))/(Resistance(n,4,i)/(Rigidity(n,1,i)/3)-Resistance(n,1,i)/(Rigidity(n,1,i))/2.2);
                Delta(n,i) = data_all(first_line_comp-1+j,first_column_comp+6);
                
            end
        end
    end
end

% Constitutive laws of the groups
data_laws_gr(:,1) = data_all(first_line_gr:last_line_gr,first_column_gr);
data_laws_gr(:,2) = data_all(first_line_gr:last_line_gr,first_column_gr+1);
data_laws_gr(:,3) = data_all(first_line_gr:last_line_gr,first_column_gr+2);
data_laws_gr(:,4) = data_all(first_line_gr:last_line_gr,first_column_gr+3);

% Constitutive laws of the equivalent springs
n = 0;
for i = 1:n_rows_tot
    clear temp
    for j = 1:nnz(any(Resistance(:,:,i),1))
        temp(size(Resistance,1)*(j-1)+1:size(Resistance,1)*j,1) = Resistance(:,j,i);
    end
    temp = sort(temp(temp>0));
    
    K_i(:,1,i) = Rigidity(:,1,i);
    
    for j = 1:size(temp)
        [r,c] = find(Resistance(:,:,i)==temp(j,1));
        if c ~= nnz(any(Resistance(:,:,i),1))
            
            data_laws_spring(i,j*2) = temp(j,1);
            K_i(:,j+1,i) = K_i(:,j,i);
            K_i(r,j+1,i) = Rigidity(r,c+1,i);
            
        else
            
            data_laws_spring(i,j*2) = temp(j,1);
            
            break
        end
    end
    
    K_i(K_i==0)=NaN;
    inv_K_i = 1./K_i;
    
    for j = 1:nnz(any(inv_K_i(:,:,i),1))
        data_laws_spring(i,j*2-1) = 1/nansum(inv_K_i(:,j,i));
    end
    
    if data_rows(i,3) == 1
        n = n+1;
        if n > 2
            for j = 1:2:nnz(any(data_laws_spring(i,:),1))
                
                data_laws_spring(i,j) = data_laws_spring(i,j)*n_demult;
                
            end
        end
    end
    
    % Maximum displacement of the equivalent spring
    [r_d,c_d] = find(Resistance(:,nnz(any(Resistance(:,:,i),1)),i)==data_laws_spring(i,nnz(any(data_laws_spring(i,:),1))));
    n_r = 1;
    Delta_spring(i,n_r) = data_laws_spring(i,2)/data_laws_spring(i,1);
    for j = 3:2:nnz(any(data_laws_spring(i,:),1))-1
        n_r = n_r+1;
        Delta_spring(i,n_r) = Delta_spring(i,n_r-1)+(data_laws_spring(i,j+1)-data_laws_spring(i,j-1))/data_laws_spring(i,j);
    end
    
    Delta_max(i,1) = Delta_spring(i,nnz(any(Delta_spring(i,:),1)))+Delta(r_d,i);
    
end

% Maximum displacement of the groups
for i = 1:n_gr_tot
    
   Delta_max(n_rows_tot+i,1) = data_laws_gr(i,3)/500000000 + data_laws_gr(i,4);
    
end

% Conversion to FINELG laws parameters
% Column 1 : Constitutive law number
% Column 2 : law type 
% Column 3 to ... : law parameters

for i = 1:size(data_laws_spring,1)
    
    meca(i,1) = i;
    meca(i,2) = 11;
    meca(i,3) = data_laws_spring(i,1);
    meca(i,4) = 0;
    
    for j = 2:2:nnz(any(data_laws_spring(i,:),1))
        
        meca(i,j+3) = data_laws_spring(i,j);
        
    end
    
    if nnz(any(data_laws_spring(i,:),1)) > 2
        
        meca(i,6) = data_laws_spring(i,2)/data_laws_spring(i,1)+(data_laws_spring(i,4)-data_laws_spring(i,2))/(data_laws_spring(i,3));
        
        for j = 5:2:nnz(any(data_laws_spring(i,:),1))
            
            meca(i,j+3) = meca(i,j-2+3) + (data_laws_spring(i,j+1)-data_laws_spring(i,j-1))/(data_laws_spring(i,j));
            
        end
        
    end
    meca(i,nnz(any(meca(i,:),1))+2) = 1;
end

if size(meca,2) > 20
    error('Equivalent spring constitutive law too nonlinear, please choose another component constitutive law type')
end

% Groups laws
for i = 1:size(data_laws_gr,1)
    
    meca(end+1,1) = meca(end,1)+1;
    meca(end,2) = 2;
    meca(end,3) = 500000000;
    meca(end,4) = 0;
    meca(end,5) = data_laws_gr(i,3);
    meca(end,6) = 1;
    
end

% Rigid material law
meca(end+1,1) = meca(end,1)+1;
meca(end,2) = 0;
meca(end,3) = 200000;
meca(end,4) = 3.0000E-01;

% "Constitutive laws" for linear constrains
% Column 1 : law number
% Column 3 to ... : linear constrain parameters

% Linear constrains for groups 1 to i
meca(end+1,1) = meca(end,1)+1;
meca(end,2) = 0;
meca(end,3) = 1;
meca(end,4) = -1;

% Linear constrains for groups 3 => n_rows_t-1 to i
meca(end+1,1) = meca(end,1)+1;
meca(end,2) = 0;
meca(end,3) = 1;
meca(end,4) = -1;
meca(end,5) = -1;
meca(end,6) = 1;

% Linear constrains for groups 2 to i
for i = 2 : (n_rows_t-1)
    
    meca(end+1,1) = meca(end,1)+1;
    meca(end,2) = 0;
    meca(end,3) = data_rows_t(1,2);
    meca(end,4) = -data_rows_t(1,2);
    meca(end,5) = -(data_rows_t(2,2)-data_rows_t(2+(i-1),2));
    meca(end,6) = data_rows_t(2,2)-data_rows_t(2+(i-1),2);
    
end

% Linear constrains for the "Bernouilli element"
x = 0;
nt = 0;
if data_rows(end,3) == 1
    for i = 1:n_rows_tot
        if x < n_rows_c+1
            if data_rows(i,3) == 1
                nt = nt+1;
            end
            if nt < 2
                
                x = x+1;
                meca(end+1,1) = meca(end,1)+1;
                meca(end,2) = 0;
                meca(end,3) = data_rows(i+1,2);
                meca(end,4) = data_rows(1,2)-data_rows(i+1,2);
                meca(end,5) = -data_rows(1,2);
                
            else
                if data_rows(i+1,3)==1
                    continue
                else
                    
                    x = x+1;
                    meca(end+1,1) = meca(end,1)+1;
                    meca(end,2) = 0;
                    meca(end,3) = data_rows(i+1,2);
                    meca(end,4) = data_rows(1,2)-data_rows(i+1,2);
                    meca(end,5) = -data_rows(1,2);
                    
                end
            end
        end
    end
else
    for i = 1:n_rows_tot
        if x < n_rows_c
            if data_rows(i,3) == 1
                nt = nt+1;
            end
            if nt < 2
                
                x = x+1;
                meca(end+1,1) = meca(end,1)+1;
                meca(end,2) = 0;
                meca(end,3) = data_rows(i+1,2);
                meca(end,4) = data_rows(1,2)-data_rows(i+1,2);
                meca(end,5) = -data_rows(1,2);
                
            else
                if data_rows(i+1,3)==1
                    continue
                else
                    
                    x = x+1;
                    meca(end+1,1) = meca(end,1)+1;
                    meca(end,2) = 0;
                    meca(end,3) = data_rows(i+1,2);
                    meca(end,4) = data_rows(1,2)-data_rows(i+1,2);
                    meca(end,5) = -data_rows(1,2);
                    
                end
            end
        else
            break
        end
    end
end

% Linear constrains for the "Bernouilli element" - Point of application of
% the axial load
meca(end+1,1) = meca(end,1)+1;
meca(end,2) = 0;
meca(end,3) = CG_beam;
meca(end,4) = data_rows(1,2)-CG_beam;
meca(end,5) = -data_rows(1,2);


%% Elements

% Column 1 : Element number
% Column 2 : Element identification number (FINELG)
% Column 3 : Constitutive law linked to the element
% Column 4 : Geometical parameter 
% Column 5 : Fist node of the element
% Column 6 : Last node of the element
% Column 18 : row active in tension (1), compression (2) or both (0)
% Column 19 : identification of groups elements

% Groups and components
nt = 0;
n_delta = 0;
for i = 1:n_rows_tot
    if data_rows(i,3) == 0
        % Rows in compression
        % Equivalent spring
        for j = data_rows(i,7) : data_rows(i,8)
            
            elements(j,1) = j;
            elements(j,2) = 201;
            elements(j,3) = i;
            if j == 1
                elements(j,4) = 1;
            else
                elements(j,4) = 0;
            end
            if i == 1
                elements(j,5) = j;
            else
                if j == data_rows(i,7)
                    elements(j,5) = data_rows(i,10);
                else
                    elements(j,5) = elements(j-1,6);
                end
            end
            elements(j,6) = elements(j,5)+1;
            elements(j,18) = 2;
        end
        
        % Identification of the node which will be usefull for the
        % displacement verification
        Delta_max(i,2) = elements(j,6);
        % Negative displacement for equivalent spring in compression
        Delta_max(i,1) = -Delta_max(i,1);
        
    else
        % Rows in tension
        nt = nt+1;
        if nt == 1
            % First row in tension
            % Groups
            n = 0;
            for j = data_rows(i,7) : data_rows(i,4)
                
                n = n+1;
                elements(j,1) = j;
                elements(j,2) = 201;
                elements(j,3) = n_rows_tot+(n_rows_t-1)-(j-data_rows(i,7));
                if j == 1
                    elements(j,4) = 1;
                else
                    elements(j,4) = 0;
                end
                if i == 1
                    elements(j,5) = j;
                else
                    if j == data_rows(i,7)
                        elements(j,5) = data_rows(i,10);
                    else
                        elements(j,5) = elements(j-1,6);
                    end
                end
                
                elements(j,6) = elements(j,5)+1;
                
                % Indentification of group element
                elements(j,19) = 1;
                
                % Identification of the node which will be usefull for the
                % displacement verification
                Delta_max(n_rows_tot+(n_rows_t-1)-(n-1),2) = elements(j,6);
                
            end
            % Equivalent spring
            for j = data_rows(i,7)+data_rows(i,4) : data_rows(i,8)
                
                elements(j,1) = j;
                elements(j,2) = 201;
                elements(j,3) = i;
                if j == 1
                    elements(j,4) = 1;
                else
                    elements(j,4) = 0;
                end
                if i == 1
                    elements(j,5) = j;
                else
                    if j == data_rows(i,7)
                        elements(j,5) = data_rows(i,10);
                    else
                        elements(j,5) = elements(j-1,6);
                    end
                end
                elements(j,6) = elements(j,5)+1;
                elements(j,18) = 1;
                
                % Identification of the node which will be usefull for the
                % displacement verification
                Delta_max(i,2) = elements(j,6);
                
            end
        elseif nt == 2
            % Second row in tension
            % Equivalent spring
            for j = data_rows(i,7) : data_rows(i,8)-data_rows(i,4)
                
                elements(j,1) = j;
                elements(j,2) = 201;
                elements(j,3) = i;
                elements(j,4) = 0;
                if j == data_rows(i,7)
                    elements(j,5) = data_rows(i,10);
                else
                    elements(j,5) = elements(j-1,6);
                end
                elements(j,6) = elements(j,5)+1;
                elements(j,18) = 1;
                
                % Identification of the node which will be usefull for the
                % displacement verification
                Delta_max(i,2) = elements(j,6);
                
            end
            % Groups
            n = 0;
            for j = data_rows(i,8)-data_rows(i,4)+1 : data_rows(i,8)
                
                n = n+1;
                elements(j,1) = j;
                elements(j,2) = 201;
                elements(j,3) = n_rows_tot+(n_rows_t-1)+(j-(data_rows(i,8)-data_rows(i,4)));
                elements(j,4) = 0;
                if j == data_rows(i,7)
                    elements(j,5) = data_rows(i,10);
                else
                    elements(j,5) = elements(j-1,6);
                end
                elements(j,6) = elements(j,5)+1;
                
                % Indentification of group element
                elements(j,19) = 1;
                
                % Identification of the node which will be usefull for the
                % displacement verification
                Delta_max(n_rows_tot+(n_rows_t-1)+n,2) = elements(j,6);
                
            end
        else
            % Other rows
            for j = 1:size(nb_groups_zone,2)
                if nt > j+2
                    % Equivalent springs in rows i > j+2 in each zone
                    for k = data_rows(i,7) : data_rows(i,7)
                        
                        elements(k,1) = k;
                        elements(k,2) = 201;
                        elements(k,3) = i;
                        elements(k,4) = 0;
                        elements(k,5) = data_rows(i,10);
                        elements(k,6) = elements(k,5)+1;
                        elements(k,18) = 1;
                        
                    end
                    for k = data_rows(i,7)+sum(data_rows_t(3:2+(j-1),4))+1 : data_rows(i,7)+sum(data_rows_t(3:2+(j-1),4))+nb_groups_zone(nt,j)
                        
                        elements(k,1) = k;
                        elements(k,2) = 201;
                        elements(k,3) = i;
                        elements(k,4) = 0;
                        if k == data_rows(i,7)
                            elements(k,5) = data_rows(i,10);
                        else
                            elements(k,5) = elements(k-1,6)+1;
                        end
                        elements(k,6) = elements(k,5)+1;
                        elements(k,18) = 1;
                        
                    end
                    if nt > j+3
                    for k = data_rows(i,7)+sum(data_rows_t(3:2+(j-1),4))+nb_groups_zone(nt,j)+1 : data_rows(i,7)+sum(data_rows_t(3:2+(j-1),4))+data_rows_t(j+2,4)
                        
                        elements(k,1) = k;
                        elements(k,2) = 201;
                        elements(k,3) = i;
                        elements(k,4) = 0;
                        elements(k,5) = elements(k-1,6);
                        elements(k,6) = elements(k,5)+1;
                        elements(k,18) = 1;
                        
                    end
                    end
                elseif nt == j+2
                    % Rows and zones including groups j+2 to i
                    % Equivalent springs
                    for k = data_rows(i,7)+sum(data_rows_t(3:nt,4))-data_rows(i,4) : 2 : data_rows(i,7)+sum(data_rows_t(3:nt,4))-data_rows(i,4)+2*data_rows(i,4)
                        
                        elements(k,1) = k;
                        elements(k,2) = 201;
                        elements(k,3) = i;
                        elements(k,4) = 0;
                        
                        if k == data_rows(i,7)
                            elements(k,5) = data_rows(i,10);
                        elseif k ~= data_rows(i,7) && k == data_rows(i,7)+sum(data_rows_t(3:nt,4))-data_rows(i,4)
                            elements(k,5) = elements(k-1,6)+1;
                        else
                            elements(k,5) = elements(k-2,6)+1;
                        end
                        elements(k,6) = elements(k,5)+1;
                        elements(k,18) = 1;
                        
                    end
                    % Groups
                    n = 0;
                    for k = data_rows(i,7)+sum(data_rows_t(3:nt,4))-data_rows(i,4)+1 : 2 : data_rows(i,7)+sum(data_rows_t(3:nt,4))-data_rows(i,4)+2*data_rows(i,4)-1
                        
                        elements(k,1) = k;
                        elements(k,2) = 201;
                        elements(k,3) = n_rows_tot+sum(data_rows(1:i,4))-(k-(k-n));
                        elements(k,4) = 0;
                        elements(k,5) = elements(k-1,6);
                        elements(k,6) = elements(k,5)+1;
                        n = n+1;
                        % Indentification of group element
                        elements(k,19) = 1;
                        
                        % Identification of the node which will be usefull for the
                        % displacement verification
                        n_delta = n_delta+1;
                        Delta_max(n_rows_tot+(n_rows_t-1)+(n_rows_t-2)+n_delta,2) = elements(k,6);
                        
                    end
                else
                    % Springs after the groups (j+2) to i
                    % Equivalent springs
                    for k = data_rows(i,7)+sum(data_rows_t(3:nt,4))+data_rows(i,4) : data_rows(i,8)
                        
                        elements(k,1) = k;
                        elements(k,2) = 201;
                        elements(k,3) = i;
                        elements(k,4) = 0;
                        elements(k,5) = elements(k-1,6);
                        elements(k,6) = elements(k,5)+1;
                        elements(k,18) = 1;
                        
                    end
                end
            end
        end
        
        % Identification of the node which will be usefull for the
        % displacement verification
        if nt > 2 && nt < n_rows_t
            Delta_max(i,2) = data_rows_t(nt,11);
        elseif nt == n_rows_t
            if data_rows(end,3)==0
                Delta_max(i,2) = data_rows_t(nt,11);
            else
                Delta_max(i,2) = data_rows_t(nt,11)-1;
            end
        end
        
    end
end

% Elements connecting embedded supports
nt = 0;
n = 0;
for i = 1:n_rows_tot
    if n < n_rows_c
        if data_rows(i,3) == 1
            nt = nt+1;
            if nt == 1
                
                n=n+1;
                elements(end+1,1) = elements(end,1)+1;
                elements(end,2) = 33;
                elements(end,3) = n_rows_tot+n_gr_tot+1;
                if i == 1
                    elements(end,4) = 1;
                else
                    elements(end,4) = 0;
                end
                elements(end,5) = data_rows(i,10);
                for j = i+1:n_rows_tot
                    if data_rows(j,3) == 0
                        elements(end,6) = data_rows(j,10);
                        break
                    end
                end
                
            else
                continue
            end
        else
            
            n=n+1;
            elements(end+1,1) = elements(end,1)+1;
            elements(end,2) = 33;
            elements(end,3) = n_rows_tot+n_gr_tot+1;
            if i == 1
                elements(end,4) = 1;
            else
                elements(end,4) = 0;
            end
            elements(end,5) = data_rows(i,10);
            for j = i+1:n_rows_tot
                if nt >= 1
                    if data_rows(j,3) == 0
                        
                        elements(end,6) = data_rows(j,10);
                        break
                        
                    end
                else
                    
                    elements(end,6) = data_rows(j,10);
                    break
                    
                end
            end
        end
    end
end

% Linear constrains elements
% Column 1 : element number
% Column 2 : element indentification number
% Column 3 : "constitutive law" of the linear constrain
% Column 5 : node linked to the linear constrain
% Column 6 to ... : linear constrain parameters

% Linear constrains for groups 1 to i
n = 0;
for i = 1:data_rows_t(1,4)
    
    elements(end+1,1) = elements(end,1)+1;
    elements(end,2) = 221;
    elements(end,3) = n_rows_tot+n_gr_tot+2;
    if i == 1
        elements(end,4) = 1;
    else
        elements(end,4) = 0;
    end
    if n < 3
        elements(end,5) = coord(data_rows(end,11),1)+1+ceil(i/3);
    else
        elements(end,5) = elements(end-1,5)+1;
    end
    elements(end,6) = data_rows_t(1,10)+i;
    elements(end,7) = data_rows_t(end-(i-1),10);
    elements(end,8) = 0;
    elements(end,9) = 0;
    elements(end,10) = 0;
    n = n+1;
    if n == 4
        n = 1;
    end
    elements(end,11) = n;
    elements(end,12) = 1;
    elements(end,13) = 1;
    
end

% Linear constrains of groups 3 => n_rows_t to i
for i = 3:n_rows_t-1
    for j = 1:nnz(any(nb_groups_zone(i,:),1))
        n_lc = 0;
        if i == j+2
            for k = sum(data_rows_t(3:2+(j-1),4)) : 2 : sum(data_rows_t(3:2+(j-1),4))+(data_rows_t(i,4)-1)*2
                
                elements(end+1,1) = elements(end,1)+1;
                elements(end,2) = 221;
                elements(end,3) = n_rows_tot+n_gr_tot+3;
                elements(end,4) = 0;
                if n < 3
                    elements(end,5) = elements(end-1,5);
                else
                    elements(end,5) = elements(end-1,5)+1;
                end
                elements(end,6) = elements(k+data_rows_t(i,7),6);
                elements(end,7) = elements((k-n_lc)+data_rows_t(i+1,7),6);
                elements(end,8) = elements(end,6)+1;
                elements(end,9) = elements(end,7)+1;
                elements(end,10) = 0;
                elements(end,11) = 0;
                elements(end,12) = 0;
                n = n+1;
                if n == 4
                    n = 1;
                end
                elements(end,13) = n;
                elements(end,14) = 1;
                elements(end,15) = 1;
                elements(end,16) = 1;
                elements(end,17) = 1;
                n_lc = n_lc+1;
                
            end
        else
            for k = sum(data_rows_t(3:2+(j-1),4)) : sum(data_rows_t(3:2+(j-1),4))+data_rows_t(i,4)-1
                
                elements(end+1,1) = elements(end,1)+1;
                elements(end,2) = 221;
                elements(end,3) = n_rows_tot+n_gr_tot+3;
                elements(end,4) = 0;
                if n < 3
                    elements(end,5) = elements(end-1,5);
                else
                    elements(end,5) = elements(end-1,5)+1;
                end
                elements(end,6) = elements(k+data_rows_t(i,7),6);
                elements(end,7) = elements(k+data_rows_t(i+1,7),6);
                elements(end,8) = elements(end,6)+1;
                elements(end,9) = elements(end,7)+1;
                elements(end,10) = 0;
                elements(end,11) = 0;
                elements(end,12) = 0;
                n = n+1;
                if n == 4
                    n = 1;
                end
                elements(end,13) = n;
                elements(end,14) = 1;
                elements(end,15) = 1;
                elements(end,16) = 1;
                elements(end,17) = 1;
            end
        end
    end
end

% Linear constrains for groups 2 to i
for i = 1:data_rows_t(2,4)
    
    elements(end+1,1) = elements(end,1)+1;
    elements(end,2) = 221;
    elements(end,3) = elements(end-1,3)+1;
    elements(end,4) = 0;
    if n < 3
        elements(end,5) = elements(end-1,5);
    else
        elements(end,5) = elements(end-1,5)+1;
    end
    elements(end,6) = data_rows_t(2,10)+i;
    elements(end,7) = data_rows_t(2+i,11);
    elements(end,8) = data_rows(1,11);
    elements(end,9) = data_rows(end,11);
    elements(end,10) = 0;
    elements(end,11) = 0;
    elements(end,12) = 0;
    n = n+1;
    if n == 4
        n = 1;
    end
    elements(end,13) = n;
    elements(end,14) = 1;
    elements(end,15) = 1;
    elements(end,16) = 1;
    elements(end,17) = 1;
    
end

if data_rows(end,3) == 1
    elements(end,7) = data_rows_t(end,11)-1;
end

% Linear constrains of the "Bernouilli element"
nt = 0;
x = 0;
if data_rows(end,3) == 1
    for i = 1:n_rows_tot
        if x < n_rows_c+1
            if data_rows(i,3) == 1
                nt = nt+1;
            end
            if nt < 2
                
                elements(end+1,1) = elements(end,1)+1;
                elements(end,2) = 221;
                elements(end,3) = elements(end-1,3)+1;
                elements(end,4) = 0;
                if n < 3
                    elements(end,5) = elements(end-1,5);
                else
                    elements(end,5) = elements(end-1,5)+1;
                end
                elements(end,6) = data_rows(1,11);
                elements(end,7) = data_rows(end,11);
                elements(end,8) = data_rows(i+1,11);
                elements(end,9) = 0;
                elements(end,10) = 0;
                elements(end,11) = 0;
                n = n+1;
                if n == 4
                    n = 1;
                end
                elements(end,12) = n;
                elements(end,13) = 1;
                elements(end,14) = 1;
                elements(end,15) = 1;
                x=x+1;
                
            else
                if data_rows(i+1,3)==1
                    continue
                else
                    elements(end+1,1) = elements(end,1)+1;
                    elements(end,2) = 221;
                    elements(end,3) = elements(end-1,3)+1;
                    elements(end,4) = 0;
                    if n < 3
                        elements(end,5) = elements(end-1,5);
                    else
                        elements(end,5) = elements(end-1,5)+1;
                    end
                    elements(end,6) = data_rows(1,11);
                    elements(end,7) = data_rows(end,11);
                    elements(end,8) = data_rows(i+1,11);
                    elements(end,9) = 0;
                    elements(end,10) = 0;
                    elements(end,11) = 0;
                    n = n+1;
                    if n == 4
                        n = 1;
                    end
                    elements(end,12) = n;
                    elements(end,13) = 1;
                    elements(end,14) = 1;
                    elements(end,15) = 1;
                    x = x+1;
                    
                end
            end
        end
    end
else
    for i = 1:n_rows_tot
        if x < n_rows_c
            if data_rows(i,3) == 1
                nt = nt+1;
            end
            if nt < 2
                
                elements(end+1,1) = elements(end,1)+1;
                elements(end,2) = 221;
                elements(end,3) = elements(end-1,3)+1;
                elements(end,4) = 0;
                if n < 3
                    elements(end,5) = elements(end-1,5);
                else
                    elements(end,5) = elements(end-1,5)+1;
                end
                elements(end,6) = data_rows(1,11);
                elements(end,7) = data_rows(end,11);
                elements(end,8) = data_rows(i+1,11);
                elements(end,9) = 0;
                elements(end,10) = 0;
                elements(end,11) = 0;
                n = n+1;
                if n == 4
                    n = 1;
                end
                elements(end,12) = n;
                elements(end,13) = 1;
                elements(end,14) = 1;
                elements(end,15) = 1;
                x=x+1;
                
            else
                if data_rows(i+1,3)==1
                    continue
                else
                    elements(end+1,1) = elements(end,1)+1;
                    elements(end,2) = 221;
                    elements(end,3) = elements(end-1,3)+1;
                    elements(end,4) = 0;
                    if n < 3
                        elements(end,5) = elements(end-1,5);
                    else
                        elements(end,5) = elements(end-1,5)+1;
                    end
                    elements(end,6) = data_rows(1,11);
                    elements(end,7) = data_rows(end,11);
                    elements(end,8) = data_rows(i+1,11);
                    elements(end,9) = 0;
                    elements(end,10) = 0;
                    elements(end,11) = 0;
                    n = n+1;
                    if n == 4
                        n = 1;
                    end
                    elements(end,12) = n;
                    elements(end,13) = 1;
                    elements(end,14) = 1;
                    elements(end,15) = 1;
                    x = x+1;
                    
                end
            end
        end
    end
end

% Linear constrain for "Bernouilli element" - Axial force application node
elements(end+1,1) = elements(end,1)+1;
elements(end,2) = 221;
elements(end,3) = elements(end-1,3)+1;
elements(end,4) = 0;
if n < 3
    elements(end,5) = elements(end-1,5);
else
    elements(end,5) = elements(end-1,5)+1;
end
elements(end,6) = data_rows(1,11);
elements(end,7) = data_rows(end,11);
elements(end,8) = coord(data_rows(end,11)+1,1);
elements(end,9) = 0;
elements(end,10) = 0;
elements(end,11) = 0;
n = n+1;
if n == 4
    n = 1;
end
elements(end,12) = n;
elements(end,13) = 1;
elements(end,14) = 1;
elements(end,15) = 1;

% Linear constrains for equalization of the displacements of all aquivalent
% springs of row i
for i = 3:n_rows_t
    for j = data_rows_t(i,7) : data_rows_t(i,8)-1
        if elements(j,19) == 0
            
            elements(end+1,1) = elements(end,1)+1;
            elements(end,2) = 221;
            elements(end,3) = n_rows_tot+n_gr_tot+3;
            elements(end,4) = 0;
            if n < 3
                elements(end,5) = elements(end-1,5);
            else
                elements(end,5) = elements(end-1,5)+1;
            end
            elements(end,6) = elements(j,6);
            elements(end,7) = elements(j,5);
            elements(end,8) = elements(data_rows_t(i,8),6);
            elements(end,9) = elements(data_rows_t(i,8),6)-1;
            elements(end,10) = 0;
            elements(end,11) = 0;
            elements(end,12) = 0;
            n = n+1;
            if n == 4
                n = 1;
            end
            elements(end,13) = n;
            elements(end,14) = 1;
            elements(end,15) = 1;
            elements(end,16) = 1;
            elements(end,17) = 1;
            
        end
    end
end

%% Supports

nt = 0;
n = 0;
nb_col = 0;
nb_row = 1;

% Embedded supports
for i = 1:n_gr_tot
    if n <= n_rows_c
        if data_rows(i,3) == 1
            nt = nt+1;
            if nt == 1
                
                n=n+1;
                if nb_col == 16
                    nb_row = nb_row+1;
                    nb_col = 1;
                else
                    nb_col = nb_col+1;
                end
                supports(nb_row,1) = 1110000;
                supports(nb_row,2) = 0;
                supports(nb_row,3) = 0;
                supports(nb_row,3+nb_col) = data_rows(i,10);
                
            else
                continue
            end
        else
            
            n=n+1;
            if nb_col == 16
                nb_row = nb_row+1;
                nb_col = 1;
            else
                nb_col = nb_col+1;
            end
            supports(nb_row,1) = 1110000;
            supports(nb_row,2) = 0;
            supports(nb_row,3) = 0;
            supports(nb_row,3+nb_col) = data_rows(i,10);
            
        end
    end
end

nb_col = 0;
nb_row = nb_row+1;

% Springs internal supports (not part of a group or an embedded support)
for i = 3:n_rows_t
    if i < n_rows_t
        for j = data_rows_t(i,7)+sum(data_rows_t(3:i-1,4))+data_rows_t(i,4)*2 : data_rows_t(i,8)-1
            
            if nb_col == 16
                nb_row = nb_row+1;
                nb_col = 1;
            else
                nb_col = nb_col+1;
            end
            supports(nb_row,1) = 0110000;
            supports(nb_row,2) = 0;
            supports(nb_row,3) = 0;
            supports(nb_row,3+nb_col) = elements(j,6);
            
        end
    end
        for j = 1 : i-4
            if i >= j+4
            for k = data_rows_t(i,7)+sum(data_rows_t(3:2+(j-1)))+nb_groups_zone(i,j) : data_rows_t(i,7)+sum(data_rows_t(3:2+j,4))-1
                
                if nb_col == 16
                    nb_row = nb_row+1;
                    nb_col = 1;
                else
                    nb_col = nb_col+1;
                end
                supports(nb_row,1) = 0110000;
                supports(nb_row,2) = 0;
                supports(nb_row,3) = 0;
                supports(nb_row,3+nb_col) = elements(k,6);
                
            end
            end
        end
end

% Groups supports
a = 0;
for i = 1:size(supports,1)
    for j = 4:size(supports,2)
        if supports(i,j) ~= 0
            a = a+1;
            nodes(a,1) = supports(i,j);
            nodes = sort(nodes);
        end
    end
end

nb_col = 0;
if nnz(any(supports,2)) == nb_row
    nb_row = nb_row+1;
end

for i = 1:size(nodes,1)-1
    if nodes(i+1,1) ~= nodes(i,1)+1
        n = nodes(i,1)+1;
        while n ~= nodes(i+1,1)
            
            if nb_col == 16
                nb_row = nb_row+1;
                nb_col = 1;
            else
                nb_col = nb_col+1;
            end
            supports(nb_row,1) = 0100000;
            supports(nb_row,2) = 0;
            supports(nb_row,3) = 0;
            supports(nb_row,3+nb_col) = n;
            n = n+1;
            
        end
    end
end

if nodes(end,1)+1 ~= data_rows(end,11)
    
    if nb_col == 16
        nb_row = nb_row+1;
        nb_col = 1;
    else
        nb_col = nb_col+1;
    end
    supports(nb_row,1) = 0100000;
    supports(nb_row,2) = 0;
    supports(nb_row,3) = 0;
    supports(nb_row,3+nb_col) = data_rows(end,11)-1;
    
    if nb_col == 16
        nb_row = nb_row+1;
        nb_col = 1;
    else
        nb_col = nb_col+1;
    end
    supports(nb_row,1) = 0100000;
    supports(nb_row,2) = 0;
    supports(nb_row,3) = 0;
    supports(nb_row,3+nb_col) = data_rows(end,11);
    
else
    
    if nb_col == 16
        nb_row = nb_row+1;
        nb_col = 1;
    else
        nb_col = nb_col+1;
    end
    supports(nb_row,1) = 0100000;
    supports(nb_row,2) = 0;
    supports(nb_row,3) = 0;
    supports(nb_row,3+nb_col) = data_rows(end,11);
    
end

% Point of application of the axial load
if nb_col == 16
    nb_row = nb_row+1;
    nb_col = 1;
else
    nb_col = nb_col+1;
end
supports(nb_row,1) = 0100000;
supports(nb_row,2) = 0;
supports(nb_row,3) = 0;
supports(nb_row,3+nb_col) = data_rows(end,11)+1;


%% Sequences

% Number of sequences
if M == 0
    % N only
    nb_seq = 1;
else
    % M only and other MN
    nb_seq = 4;
end

% FINELG input
for i = 1:nb_seq
    sequences(i,1) = 1;
    sequences(i,2) = 1;
    for j = 3:6
        if j-2 == i
            sequences(i,j) = 1;
        else
            sequences(i,j) = 0;
        end
    end
end


%% Loading

% Number of loads
if M == 0
    % N only
    nb_loads = 1;
else
    % M only and other MN
    nb_loads = 7;
end

% FINELG input
for i = 1:nb_loads
    
    loading(i,1) = i;
    loading(i,2) = 0;
    loading(i,4) = 0;
    
end
for i = nb_loads+1:nb_loads*2
    
    loading(i,2) = 1;
    loading(i,3) = i-nb_loads;
    loading(i,4) = 0;
    
end

if M == 0
    
    loading(nb_loads,3) = N;
    loading(nb_loads*2,1) = 1;
    loading(nb_loads*2,5) = data_rows(end,11)+1;
    
else
    loading(1,3) = N1;
    if M < 0
        loading(2,3) = M1/data_rows(1,2);
        loading(3,3) = -M1/data_rows(1,2);
    elseif M > 0
        loading(2,3) = -M1/data_rows(1,2);
        loading(3,3) = M1/data_rows(1,2);
    end
    loading(4,3) = -N1;
    loading(5,3) = -M/data_rows(1,2);
    loading(6,3) = M/data_rows(1,2);
    loading(7,3) = N;
    
    loading(8,1) = 1;
    loading(9,1) = 2;
    loading(10,1) = 2;
    loading(11,1) = 3;
    loading(12,1) = 4;
    loading(13,1) = 4;
    loading(14,1) = 4;
    
    loading(8,5) = data_rows(end,11)+1;
    loading(9,5) = data_rows(1,11);
    loading(10,5) = data_rows(end,11);
    loading(11,5) = data_rows(end,11)+1;
    loading(12,5) = data_rows(1,11);
    loading(13,5) = data_rows(end,11);
    loading(14,5) = data_rows(end,11)+1;
end


%% Control

control(1,1) = 0;
control(1,2) = 0;
control(1,3) = 0;
control(1,4) = 23;
control(1,5) = 1200060;
control(1,6) = 0;
control(1,7) = 1;
control(1,8) = 0;
control(1,9) = 0;
control(1,10) = 0;
control(1,11) = 0;

for i = 2:1+nb_seq
    
    control(i,1) = 0;
    control(i,2) = 0;
    control(i,3) = 500;
    control(i,4) = 1;
    control(i,5) = 99999999;
    
end

control(end,1) = 1;

%% Data writing

FileName = (fullfile([Name, Ext]));
nb_files = 0;

if exist(FileName,'file')
    
    Dir = dir(fullfile([Name, '*', Ext]));
    nb_files = size(Dir,1);
    FileName = fullfile([Name, sprintf('%d', nb_files), Ext]);
    
end

fid = fopen(FileName,'w');

% Version FINELG
fprintf(fid,'%s%6s%8s\n','FINELG','103','12');

% Control
fprintf(fid,'%s\n','CTRL');
fprintf(fid,'%4s%4s\n','N','MM');
for i = 1:3
    fprintf(fid,'%4.0f',control(1,i));
end
for i = 4:5
    fprintf(fid,'%8.0f',control(1,i));
end
for i = 6:size(control,2)
    fprintf(fid,'%4.0f',control(1,i));
end
fprintf(fid,'\n%8s\n','NONL');
for i = 2:size(control,1)
    fprintf(fid,'%8s%4.0f\n','SEQP',i-1);
    fprintf(fid,'%4.0f%4.0f%4.0f%24.0f%12.0f\n\n',control(i,1:5));
end
fprintf(fid,'%s\n','CTRL_END');

% Loading sequences
for i = 1:size(sequences,1)-1
    fprintf(fid,'%s%8.0f\n','SEQP',i);
    fprintf(fid,'%8s\n%4.0f%12.3f%8.3f%8.3f%8.3f%8.3f\n','COMB',sequences(i,1:6));
    fprintf(fid,'%8s\n%4.0f%4.0f\n','INCR',110,-4);
    fprintf(fid,'%8s\n%8.3f\n','CREM',0.1);
    fprintf(fid,'%8s\n%4.0f%4.0f%4.0f\n','MOPA',1,60,-4);
    fprintf(fid,'%8s\n%4.0f\n%4.0f\n','NODC',data_rows(end,11)+1,1);
    fprintf(fid,'%s%4.0f\n','SEQP_END',i);
end
fprintf(fid,'%s%8.0f\n','SEQP',nb_seq);
fprintf(fid,'%8s\n%4.0f%12.3f%8.3f%8.3f%8.3f%8.3f\n','COMB',sequences(end,1:6));
fprintf(fid,'%8s\n%4.0f%4.0f%4.0f\n','INCR',-4,139,4);
fprintf(fid,'%8s\n%8.3f\n','CREM',incr_ini);
fprintf(fid,'%8s\n%4.0f%4.0f%4.0f\n','MOPA',1,60,-4);
fprintf(fid,'%8s\n%4.0f\n%4.0f\n','NODC',data_rows(end,11)+1,1);
fprintf(fid,'%s%4.0f\n','SEQP_END',nb_seq);

% Meca
fprintf(fid,'%s','MECA');
for i = 1:n_rows_tot+n_gr_tot
    
    nb_lines = ceil((nnz(any(meca(i,3:end),1))+1)/6);
    nb_terms = -((nb_lines-1)*8-nnz(any(meca(i,:),1))+1);
    
    if nnz(any(meca(i,:),1))+1 <= 8
        fprintf(fid,'\n%4.0f%4.0f%12.2f%12.8f%12.2f%12.8f%12.2f%12.8f',meca(i,1:nnz(any(meca(i,:),1))+1));
    else
        fprintf(fid,'\n%4.0f%4.0f%12.2f%12.8f%12.2f%12.8f%12.2f%12.8f',meca(i,1:8));
        for j = 2:nb_lines-1
            fprintf(fid,'\n%4.0f%16.2f%12.8f%12.2f%12.8f%12.2f%12.8f',-meca(i,1),meca(i,(8+(j-2)*6+1):(8+(j-1)*6)));
        end
        fprintf(fid,'\n%4.0f%16.2f%12.8f%12.2f%12.8f%12.2f%12.8f',-meca(i,1),meca(i,(8+(nb_lines-2)*6+1):nnz(any(meca(i,:),1))+1));
    end 
end
for i = n_rows_tot+n_gr_tot+1:size(meca,1)
    fprintf(fid,'\n%4.0f%4.0f%12.3f%12.3f%12.3f%12.3f',meca(i,1:nnz(any(meca(i,:),1))+1));
end
fprintf(fid,'\n%s\n','MECA_END');

% Geometry
fprintf(fid,'%s\n','GEOM');
fprintf(fid,'%4.0f%4.0f%12.2f%12.0f\n',1,31,90,67500);
fprintf(fid,'%s\n','GEOM_END');

% Coordinates
fprintf(fid,'%s\n','COOR');
for i = 1:data_rows(end,11)+1
    fprintf(fid,'%4.0f%4.0f%12.3f%12.3f%12.3f\n',coord(i,1:5));
end
for i = data_rows(end,11)+2:size(coord,1)
    fprintf(fid,'%4.0f\n',coord(i,1));
end

% Elements
fprintf(fid,'%s\n','ELEM');
for i = 1:data_rows(end,8)+n_rows_c
    if i == 1 && elements(i,18) ~= 0
        fprintf(fid,'%4.0f%4.0f%4.0f%4.0f%4.0f%4.0f%44s%5.0f\n',elements(i,1:6),'S',elements(i,18));
    elseif i == 1 && elements(i,18) == 0
        fprintf(fid,'%4.0f%4.0f%4.0f%4.0f%4.0f%4.0f%44s\n',elements(i,1:6),'S');
    elseif elements(i,18) == 0
        fprintf(fid,'%4.0f%4.0f%4.0f%4.0f%4.0f%4.0f\n',elements(i,1:6));
    else
        fprintf(fid,'%4.0f%4.0f%4.0f%4.0f%4.0f%4.0f%49.0f\n',elements(i,1:6),elements(i,18));
    end
end
for i = data_rows(end,8)+n_rows_c+1:data_rows(end,8)+n_rows_c+data_rows_t(1,4)
    for j = 1:12
        fprintf(fid,'%4.0f',elements(i,j));
    end
    fprintf(fid,'%4.0f\n',elements(i,13));
end
for i = data_rows(end,8)+n_rows_c+data_rows_t(1,4)+1:data_rows(end,8)+n_rows_c+data_rows_t(1,4)+data_rows_t(2,4)+sum(lin_const_int)
    for j = 1:16
        fprintf(fid,'%4.0f',elements(i,j));
    end
    fprintf(fid,'%3.0f\n%20.0f\n',elements(i,17),1);
end
if data_rows(end,3) == 0
for i = data_rows(end,8)+n_rows_c+data_rows_t(1,4)+data_rows_t(2,4)+sum(lin_const_int)+1:data_rows(end,8)+n_rows_c+data_rows_t(1,4)+data_rows_t(2,4)+sum(lin_const_int)+n_rows_c+1
    for j = 1:14
        fprintf(fid,'%4.0f',elements(i,j));
    end
    fprintf(fid,'%4.0f\n',elements(i,15));
end
for i = data_rows(end,8)+n_rows_c+data_rows_t(1,4)+data_rows_t(2,4)+sum(lin_const_int)+n_rows_c+2:size(elements,1)
    for j = 1:16
        fprintf(fid,'%4.0f',elements(i,j));
    end
    fprintf(fid,'%3.0f\n%20.0f\n',elements(i,17),1);
end
else
for i = data_rows(end,8)+n_rows_c+data_rows_t(1,4)+data_rows_t(2,4)+sum(lin_const_int)+1:data_rows(end,8)+n_rows_c+data_rows_t(1,4)+data_rows_t(2,4)+sum(lin_const_int)+n_rows_c+2
    for j = 1:14
        fprintf(fid,'%4.0f',elements(i,j));
    end
    fprintf(fid,'%4.0f\n',elements(i,15));
end 
for i = data_rows(end,8)+n_rows_c+data_rows_t(1,4)+data_rows_t(2,4)+sum(lin_const_int)+n_rows_c+3:size(elements,1)
    for j = 1:16
        fprintf(fid,'%4.0f',elements(i,j));
    end
    fprintf(fid,'%3.0f\n%20.0f\n',elements(i,17),1);
end
end

fprintf(fid,'%s\n','RENU');
fprintf(fid,'%s\n','ELEM_END');

% Loading
fprintf(fid,'%s\n','CHAR');
for i = 1:nb_loads
    fprintf(fid,'%4.0f%4.0f%12.0f%12.0f\n',loading(i,1:4));
end
fprintf(fid,'%7s\n','CAS');
for i = nb_loads+1:size(loading,1)
    for j = 1:size(loading,2)-1
        fprintf(fid,'%4.0f',loading(i,j));
    end
    fprintf(fid,'%4.0f\n',loading(i,5));
end
fprintf(fid,'%s\n','CHAR_END');

% Supports
fprintf(fid,'%8s\n','APPU');
for i = 1:size(supports,1)
    fprintf(fid,'%8.0f',supports(i,1));
    for j = 2:size(supports,2)-1
        fprintf(fid,'%4.0f',supports(i,j));
    end
    fprintf(fid,'%4.0f\n',supports(i,size(supports,2)));
end
fprintf(fid,'%s\n','END');

fclose(fid);


if nb_files > 0
    
    FileName_txt = fullfile([Name_txt, sprintf('%d', nb_files), Ext_txt]);
    
else
    
    FileName_txt = (fullfile([Name_txt, Ext_txt]));
    
end

% Text file for the verification of the displacement + loads in results processing

fid = fopen(FileName_txt,'w');

fprintf(fid,'%20.3f%20.3f%20.3f\n',M,N,V);
for i = 1:size(Delta_max,1)
    fprintf(fid,'%20.6f%20.0f\n',Delta_max(i,1),Delta_max(i,2));
end

fclose(fid);